<?php

/**
 * Netifyd class.
 *
 * @category   apps
 * @package    netify
 * @subpackage libraries
 * @author     eGloo <team@egloo.ca>
 * @copyright  2016 eGloo
 * @license    http://www.gnu.org/copyleft/lgpl.html GNU Lesser General Public License version 3 or later
 * @link       https://www.egloo.ca/netify/community
 */

///////////////////////////////////////////////////////////////////////////////
// N A M E S P A C E
///////////////////////////////////////////////////////////////////////////////

namespace clearos\apps\netify;

///////////////////////////////////////////////////////////////////////////////
// B O O T S T R A P
///////////////////////////////////////////////////////////////////////////////

$bootstrap = getenv('CLEAROS_BOOTSTRAP') ? getenv('CLEAROS_BOOTSTRAP') : '/usr/clearos/framework/shared';
require_once $bootstrap . '/bootstrap.php';

///////////////////////////////////////////////////////////////////////////////
// T R A N S L A T I O N S
///////////////////////////////////////////////////////////////////////////////

clearos_load_language('netify');

///////////////////////////////////////////////////////////////////////////////
// D E P E N D E N C I E S
///////////////////////////////////////////////////////////////////////////////

// Classes
//--------

use \clearos\apps\base\Daemon as Daemon;
use \clearos\apps\base\File as File;

clearos_load_library('base/Daemon');
clearos_load_library('base/File');

///////////////////////////////////////////////////////////////////////////////
// C L A S S
///////////////////////////////////////////////////////////////////////////////

/**
 * Netifyd class.
 *
 * @category   apps
 * @package    netify
 * @subpackage libraries
 * @author     eGloo <team@egloo.ca>
 * @copyright  2016 eGloo
 * @license    http://www.gnu.org/copyleft/lgpl.html GNU Lesser General Public License version 3 or later
 * @link       https://www.egloo.ca/netify/community
 */

class Netifyd extends Daemon
{

    ///////////////////////////////////////////////////////////////////////////////
    // C O N S T A N T S
    ///////////////////////////////////////////////////////////////////////////////

    const FILE_CONFIG = '/etc/netifyd.conf';
    const FILE_CACHE = '/var/clearos/netify/cache';

    ///////////////////////////////////////////////////////////////////////////////
    // V A R I A B L E S
    ///////////////////////////////////////////////////////////////////////////////

    protected $loaded = FALSE;
    protected $config = array();

    /**
     * Netifyd constructor.
     */

    public function __construct()
    {
        clearos_profile(__METHOD__, __LINE__);

        parent::__construct('netifyd');
    }

    /**
     * Returns site UUID.
     *
     * @return string site UUID
     * @throws Engine_Exception
     */

    public function get_uuid()
    {
        clearos_profile(__METHOD__, __LINE__);

        $this->_load_config();

        if (!empty($this->config['netifyd']['uuid']))
            return $this->config['netifyd']['uuid'];
        else
            return '';
    }

    /**
     * Returns sink.
     *
     * @return string sink
     * @throws Engine_Exception
     */

    public function get_sink()
    {
        clearos_profile(__METHOD__, __LINE__);

        $this->_load_config();

        if (!empty($this->config['netifyd']['enable_netify_sink']))
            return $this->config['netifyd']['enable_netify_sink'];
        else
            return FALSE;
    }

    /**
     * Returns MAC whitelist.
     *
     * @return array mac addresses to whitelist
     * @throws Engine_Exception
     */

    public function get_whitelist()
    {
        clearos_profile(__METHOD__, __LINE__);

        $this->_load_config();

        if (!empty($this->config['filter']['mac']))
            return $this->config['filter']['mac'];
        else
            return [];
    }

    /**
     * Delete a MAC from whitelist.
     *
     * @return void
     * @throws Engine_Exception
     */

    public function delete_mac($mac)
    {
        clearos_profile(__METHOD__, __LINE__);

        $file = new File(self::FILE_CONFIG, TRUE);
        $file->delete_lines("/^mac\\[.+\\]\s+=\s+$mac/");
    }

    /**
     * Set sink.
     *
     * @param string $sink sink
     *
     * @return void
     * @throws Engine_Exception
     */

    public function set_sink($sink)
    {
        clearos_profile(__METHOD__, __LINE__);

        if ($sink === 'on' || $sink == 1 || $sink == TRUE)
            $sink = 'yes';
        else
            $sink = 'no';

        $this->_set_parameter('enable_netify_sink', $sink);
    }

    /**
     * Loads configuration.
     *
     * @return void
     * @throws Engine_Exception
     */

    private function _load_config()
    {
        clearos_profile(__METHOD__, __LINE__);

        if ($this->loaded)
            return;

        $config = new File(self::FILE_CONFIG, TRUE);
        $cache = new File(self::FILE_CACHE);

        if (! $cache->exists()) {
            $config->copy_to(self::FILE_CACHE);
            $config->chown('webconfig', 'webconfig');
        }

        $this->config = parse_ini_file(self::FILE_CACHE, TRUE);

        $cache->delete();

        $this->loaded = TRUE;
    }

    /**
     * Generic set routine.
     *
     * @param string $key   key name
     * @param string $value value for the key
     *
     * @return  void
     * @throws Engine_Exception
     */

    function _set_parameter($key, $value)
    {
        clearos_profile(__METHOD__, __LINE__);

        try {
            $file = new File(self::FILE_CONFIG, TRUE);
            $match = $file->replace_lines("/^$key\s*=\s*/", "$key = $value\n");

            if (!$match)
                $file->add_lines("$key = $value\n");
        } catch (Exception $e) {
            throw new Engine_Exception(clearos_exception_message($e), CLEAROS_ERROR);
        }

        $this->loaded = FALSE;
    }

    ///////////////////////////////////////////////////////////////////////////////
    // V A L I D A T I O N   R O U T I N E S
    ///////////////////////////////////////////////////////////////////////////////

    /**
     * Validation routine for sink.
     *
     * @param string  $sink       sink
     *
     * @return boolean error message if sink is invalid
     */

    public function validate_sink($sink)
    {
        clearos_profile(__METHOD__, __LINE__);
        // Set as 'yes' or 'no', regardless of value
    }

}
